@@ -163,13 +163,22 @@ class Model
             }
         }
 
+        $doneFlag = Rules::getDoneFlagArchiveContainsAllPlugins($segment ?: new Segment('', []));
+
         if (empty($plugin)) {
-            $doneFlag = Rules::getDoneFlagArchiveContainsAllPlugins($segment ?: new Segment('', []));
+            if (null === $segment) {
+                $nameCondition = "name LIKE '$doneFlag%'"; // invalidate all segments
+            } else {
+                $nameCondition = "(name = '$doneFlag' OR name LIKE '$doneFlag.%')"; // invalidate specific segment only
+            }
         } else {
-            $doneFlag = Rules::getDoneFlagArchiveContainsOnePlugin($segment ?: new Segment('', []), $plugin);
+            if (null === $segment) {
+                $nameCondition = "name LIKE '$doneFlag%.$plugin'"; // invalidate all segments for specific plugin
+            } else {
+                $nameCondition = "name = '$doneFlag.$plugin'"; // invalidate specific segment for specific plugin only
+            }
         }
 
-        $nameCondition = "name LIKE '$doneFlag%'";
 
         $sql .= " AND $nameCondition";
 
@@ -187,7 +196,7 @@ class Model
 
                 // set status to DONE_INVALIDATED for finished archives
                 $sql = "UPDATE `$archiveTable` SET `value` = " . ArchiveWriter::DONE_INVALIDATED . " WHERE idarchive IN ("
-                    . implode(',', $idArchives) . ") AND value != " . ArchiveWriter::DONE_ERROR . " AND $nameCondition";
+                    . implode(',', $idArchives) . ") AND value NOT IN (" . ArchiveWriter::DONE_ERROR . ", " . ArchiveWriter::DONE_ERROR_INVALIDATED . ") AND $nameCondition";
 
                 Db::query($sql);
 
@@ -199,7 +208,7 @@ class Model
             }
         }
 
-        if (true === $doNotCreateInvalidations) {
+        if ($doNotCreateInvalidations) {
             return count($idArchives);
         }
 
@@ -219,6 +228,13 @@ class Model
             return Segment::getSegmentHash($definition);
         }, $hashesOfAllSegmentsToArchiveInCoreArchive);
 
+
+        if (empty($plugin)) {
+            $doneFlag = Rules::getDoneFlagArchiveContainsAllPlugins($segment ?: new Segment('', []));
+        } else {
+            $doneFlag = Rules::getDoneFlagArchiveContainsOnePlugin($segment ?: new Segment('', []), $plugin);
+        }
+
         $dummyArchives = [];
         foreach ($idSites as $idSite) {
             try {
@@ -321,10 +337,9 @@ class Model
      * @param int[] $idSites
      * @param string[][] $datesByPeriodType
      * @param Segment $segment
-     * @return \Zend_Db_Statement
      * @throws Exception
      */
-    public function updateRangeArchiveAsInvalidated($archiveTable, $idSites, $allPeriodsToInvalidate, ?Segment $segment = null)
+    public function updateRangeArchiveAsInvalidated($archiveTable, $idSites, $allPeriodsToInvalidate, ?Segment $segment = null): void
     {
         if (empty($idSites)) {
             return;
@@ -347,18 +362,36 @@ class Model
             }
         }
 
-        if ($segment) {
-            $nameCondition = "name LIKE '" . Rules::getDoneFlagArchiveContainsAllPlugins($segment) . "%'";
-        } else {
+        if (null === $segment) {
             $nameCondition = "name LIKE 'done%'";
+        } else {
+            $doneFlag = Rules::getDoneFlagArchiveContainsAllPlugins($segment);
+            $nameCondition = "(name = '$doneFlag' OR name LIKE '$doneFlag.%')";
         }
 
-        $sql = "UPDATE $archiveTable SET value = " . ArchiveWriter::DONE_INVALIDATED
+        $sql = "SELECT idarchive FROM $archiveTable "
              . " WHERE $nameCondition
                    AND idsite IN (" . implode(", ", $idSites) . ")
                    AND (" . implode(" OR ", $periodConditions) . ")";
 
-        return Db::query($sql, $bind);
+        $recordsToUpdate = Db::fetchAll($sql, $bind);
+
+        if (empty($recordsToUpdate)) {
+            return;
+        }
+
+        $idArchives = array_map('intval', array_column($recordsToUpdate, 'idarchive'));
+
+        $updateSql = "UPDATE $archiveTable SET value = " . ArchiveWriter::DONE_INVALIDATED
+            . " WHERE idarchive IN (" . implode(', ', $idArchives) . ") AND $nameCondition"
+            . " AND value NOT IN (" . ArchiveWriter::DONE_ERROR . ", " . ArchiveWriter::DONE_ERROR_INVALIDATED . ")";
+
+        Db::query($updateSql);
+
+        $updateSql = "UPDATE $archiveTable SET value = " . ArchiveWriter::DONE_ERROR_INVALIDATED
+                   . " WHERE idarchive IN (" . implode(', ', $idArchives) . ") AND $nameCondition AND value = " . ArchiveWriter::DONE_ERROR;
+
+        Db::query($updateSql);
     }
 
     public function getTemporaryArchivesOlderThan($archiveTable, $purgeArchivesOlderThan)
@@ -713,54 +746,16 @@ class Model
         $table = Common::prefixTable('archive_invalidations');
 
         // set archive value to in progress if not set already
-        $statement = Db::query("UPDATE `$table` SET `status` = ?, ts_started = NOW() WHERE idinvalidation = ? AND status = ?", [
+        $statement = Db::query("UPDATE `$table` SET `status` = ?, `processing_host` = ?, `process_id` = ?, `ts_started` = NOW() WHERE `idinvalidation` = ? AND `status` = ?", [
             ArchiveInvalidator::INVALIDATION_STATUS_IN_PROGRESS,
+            gethostname() ?: null,
+            Common::getProcessId(),
             $invalidation['idinvalidation'],
             ArchiveInvalidator::INVALIDATION_STATUS_QUEUED,
         ]);
 
-        if ($statement->rowCount() > 0) { // if we updated, then we've marked the archive as started
-            return true;
-        }
-
-        // archive was not originally started or was started within the expected time, so we assume it's ongoing and another process
-        // (on this machine or another) is actively archiving it.
-        $archiveFailureRecoveryTimeout = GeneralConfig::getConfigValue('archive_failure_recovery_timeout', $invalidation['idsite']);
-        if (
-            empty($invalidation['ts_started'])
-            || $invalidation['ts_started'] > Date::now()->subSeconds($archiveFailureRecoveryTimeout)->getTimestamp()
-        ) {
-            return false;
-        }
-
-        // archive was started over 24 hours ago, we assume it failed and take it over
-        Db::query("UPDATE `$table` SET `status` = ?, ts_started = NOW() WHERE idinvalidation = ?", [
-            ArchiveInvalidator::INVALIDATION_STATUS_IN_PROGRESS,
-            $invalidation['idinvalidation'],
-        ]);
-
-        // remove similar invalidations w/ lesser idinvalidation values
-        $bind = [
-            $invalidation['idsite'],
-            $invalidation['period'],
-            $invalidation['date1'],
-            $invalidation['date2'],
-            $invalidation['name'],
-            ArchiveInvalidator::INVALIDATION_STATUS_IN_PROGRESS,
-        ];
-
-        if (empty($invalidation['report'])) {
-            $reportClause = "(report IS NULL OR report = '')";
-        } else {
-            $reportClause = "report = ?";
-            $bind[] = $invalidation['report'];
-        }
-
-        $sql = "DELETE FROM " . Common::prefixTable('archive_invalidations') . " WHERE idinvalidation < ? AND idsite = ? AND "
-            . "date1 = ? AND date2 = ? AND `period` = ? AND `name` = ? AND $reportClause";
-        Db::query($sql, $bind);
-
-        return true;
+        // if we updated, then we've marked the archive as started
+        return $statement->rowCount() > 0;
     }
 
     public function isSimilarArchiveInProgress($invalidation)
@@ -789,19 +784,37 @@ class Model
         return !empty($result);
     }
 
-    public function getInvalidationsInProgress(?int $idSite = null): array
-    {
+    public function getInvalidationsInProgress(
+        array $idSites = [],
+        array $processingHosts = [],
+        ?Date $startTime = null,
+        ?Date $endTime = null
+    ): array {
         $table = Common::prefixTable('archive_invalidations');
 
         $bind = [ArchiveInvalidator::INVALIDATION_STATUS_IN_PROGRESS];
-        $idSiteCondition = '';
+        $whereConditions = '';
 
-        if (!empty($idSite)) {
-            $idSiteCondition = ' AND idsite = ?';
-            $bind[] = $idSite;
+        if (!empty($processingHosts)) {
+            $whereConditions .= sprintf(' AND `processing_host` IN (%1$s)', Common::getSqlStringFieldsArray($processingHosts));
+            $bind = array_merge($bind, $processingHosts);
         }
 
-        $sql = "SELECT idinvalidation, idsite, period, date1, date2, name, ts_started FROM `$table` WHERE `status` = ? $idSiteCondition AND ts_started IS NOT NULL ORDER BY ts_started ASC";
+        if (!empty($idSites)) {
+            $whereConditions .= sprintf(' AND `idsite` IN (' . implode(', ', $idSites) . ')');
+        }
+
+        if (!empty($startTime)) {
+            $whereConditions .= ' AND `ts_started` > ?';
+            $bind[] = $startTime->toString('Y-m-d H:i:s');
+        }
+
+        if (!empty($endTime)) {
+            $whereConditions .= ' AND `ts_started` < ?';
+            $bind[] = $endTime->toString('Y-m-d H:i:s');
+        }
+
+        $sql = "SELECT idinvalidation, idsite, period, date1, date2, name, report, ts_invalidated, ts_started, processing_host, process_id FROM `$table` WHERE `status` = ? $whereConditions AND ts_started IS NOT NULL ORDER BY ts_started ASC";
         return Db::fetchAll($sql, $bind);
     }
 
@@ -947,11 +960,12 @@ class Model
     public function hasInvalidationForPeriodAndName($idSite, Period $period, $doneFlag, $report = null)
     {
         $table = Common::prefixTable('archive_invalidations');
+        $report = (!empty($report) && !is_array($report)) ? [$report] : $report;
 
         if (empty($report)) {
             $sql = "SELECT idinvalidation FROM `$table` WHERE idsite = ? AND date1 = ? AND date2 = ? AND `period` = ? AND `name` = ?  AND `report` IS NULL LIMIT 1";
         } else {
-            $sql = "SELECT idinvalidation FROM `$table` WHERE idsite = ? AND date1 = ? AND date2 = ? AND `period` = ? AND `name` = ? AND `report` = ? LIMIT 1";
+            $sql = "SELECT idinvalidation FROM `$table` WHERE idsite = ? AND date1 = ? AND date2 = ? AND `period` = ? AND `name` = ? AND `report` IN (" . Common::getSqlStringFieldsArray($report) . ") LIMIT 1";
         }
 
         $bind = [
@@ -963,7 +977,7 @@ class Model
         ];
 
         if (!empty($report)) {
-            $bind[] = $report;
+            $bind = array_merge($bind, $report);
         }
 
         $idInvalidation = Db::fetchOne($sql, $bind);
@@ -1006,11 +1020,85 @@ class Model
         return "idsite IN ($idSitesStr) AND";
     }
 
-    public function releaseInProgressInvalidation($idinvalidation)
+    /**
+     * Releases in progress invalidations for the given ids
+     *
+     * To avoid duplicate invalidations in the database, the method is also meant to prevent having duplicates after a reset
+     * Therefor below code will check if any of the invalidations to be reset should be removed instead
+     * An invalidation can be safely removed
+     *  - if there exists another queued invalidation with the same parameters
+     *  - if there is another running invalidation, that had been started after the current one was invalidated
+     * Otherwise the invalidation will be reset
+     *
+     * @param array $idinvalidations
+     * @return int
+     * @throws \Zend_Db_Statement_Exception
+     */
+    public function releaseInProgressInvalidations(array $idinvalidations): int
     {
+        $idinvalidations = array_map('intval', $idinvalidations);
         $table = Common::prefixTable('archive_invalidations');
-        $sql = "UPDATE $table SET status = " . ArchiveInvalidator::INVALIDATION_STATUS_QUEUED . ", ts_started = NULL WHERE idinvalidation = ?";
-        Db::query($sql, [$idinvalidation]);
+        $changedCount = 0;
+
+        $sql = "SELECT * FROM $table WHERE idinvalidation IN (" . implode(',', $idinvalidations) . ")";
+        $invalidations = Db::fetchAll($sql);
+
+        // Check invalidations one by one, to ensure we safely remove invalidations in cases where two identical ones are requested to reset
+        foreach ($invalidations as $invalidation) {
+            // Look for other identical invalidations that are either not started or started after the current one had been invalidated
+            $query = "SELECT COUNT(*) FROM $table WHERE name = ? AND idsite = ? AND date1 = ? AND date2 = ? AND period = ? AND "
+                   . "(status = ? OR (status = ? AND ts_started > ?)) AND idinvalidation != ?";
+            $bind = [
+                $invalidation['name'],
+                $invalidation['idsite'],
+                $invalidation['date1'],
+                $invalidation['date2'],
+                $invalidation['period'],
+                ArchiveInvalidator::INVALIDATION_STATUS_QUEUED,
+                ArchiveInvalidator::INVALIDATION_STATUS_IN_PROGRESS,
+                $invalidation['ts_invalidated'],
+                $invalidation['idinvalidation'],
+            ];
+
+            if (empty($invalidation['report'])) {
+                $query .= " AND (report IS NULL OR report = '')";
+            } else {
+                $query .= " AND report = ?";
+                $bind[] = $invalidation['report'];
+            }
+
+            $count = Db::fetchOne($query, $bind);
+
+            if ($count > 0) {
+                $this->logger->info(
+                    'Found duplicate invalidation for params (name = {name}, idsite = {idsite}, date1 = {date1}, date2 = {date2}, period = {period}, report = {report}). Removing invalidation {idinvalidation} instead of resetting it.',
+                    $invalidation
+                );
+
+                $sql = "DELETE FROM $table WHERE status = ? AND idinvalidation = ?";
+
+                $bind = [
+                    ArchiveInvalidator::INVALIDATION_STATUS_IN_PROGRESS,
+                    $invalidation['idinvalidation'],
+                ];
+
+                $query = Db::query($sql, $bind);
+                $changedCount += $query->rowCount();
+            } else {
+                $sql = "UPDATE $table SET status = ?, processing_host = NULL, process_id = NULL, ts_started = NULL WHERE status = ? AND idinvalidation = ?";
+
+                $bind = [
+                    ArchiveInvalidator::INVALIDATION_STATUS_QUEUED,
+                    ArchiveInvalidator::INVALIDATION_STATUS_IN_PROGRESS,
+                    $invalidation['idinvalidation'],
+                ];
+
+                $query = Db::query($sql, $bind);
+                $changedCount += $query->rowCount();
+            }
+        }
+
+        return $changedCount;
     }
 
     public function resetFailedArchivingJobs()
@@ -1030,16 +1118,7 @@ class Model
             return 0;
         }
 
-        $table = Common::prefixTable('archive_invalidations');
-        $sql = "UPDATE $table SET status = ? WHERE status = ? AND idinvalidation IN (" . implode(',', $idsToReset) . ")";
-
-        $bind = [
-            ArchiveInvalidator::INVALIDATION_STATUS_QUEUED,
-            ArchiveInvalidator::INVALIDATION_STATUS_IN_PROGRESS
-        ];
-
-        $query = Db::query($sql, $bind);
-        return $query->rowCount();
+        return $this->releaseInProgressInvalidations($idsToReset);
     }
 
     public function getRecordsContainedInArchives(Date $archiveStartDate, array $idArchives, $requestedRecords): array
