@@ -99,6 +99,11 @@ class Segment
      */
     private $isSegmentEncoded;
 
+    /**
+     * @var Exception|null
+     */
+    private $missingDatesException = null;
+
     /**
      * Truncate the Segments to 8k
      */
@@ -125,7 +130,6 @@ class Segment
      */
     public function __construct($segmentCondition, $idSites, ?Date $startDate = null, ?Date $endDate = null)
     {
-
         $this->segmentQueryBuilder = StaticContainer::get('Piwik\DataAccess\LogQueryBuilder');
 
         $segmentCondition = trim($segmentCondition ?: '');
@@ -177,6 +181,25 @@ class Segment
         }
     }
 
+    /**
+     * Checks if the provided segmentCondition is valid and available for the given idSites
+     *
+     * @param string $segmentCondition
+     * @params array $idSites
+     * @return bool
+     * @api since Matomo 5.3.0
+     */
+    public static function isAvailable(string $segmentCondition, array $idSites): bool
+    {
+        try {
+            new self($segmentCondition, $idSites);
+        } catch (Exception $e) {
+            return false;
+        }
+
+        return true;
+    }
+
     /**
      * Returns the segment expression.
      * @return SegmentExpression
@@ -335,8 +358,7 @@ class Segment
 
         if ($requiresSubQuery && empty($this->startDate) && empty($this->endDate)) {
             if (Development::isEnabled()) {
-                $e = new Exception();
-                Log::warning("Avoiding segment subquery due to missing start date and/or an end date. Please ensure a start date and/or end date is set when initializing a segment if it's used to build a query. Stacktrace:\n" . $e->getTraceAsString());
+                $this->missingDatesException = new Exception();
             }
             return false;
         }
@@ -586,6 +608,14 @@ class Segment
      */
     public function getSelectQuery($select, $from, $where = false, $bind = array(), $orderBy = false, $groupBy = false, $limit = 0, $offset = 0, $forceGroupBy = false)
     {
+        if (Development::isEnabled() && !empty($this->missingDatesException)) {
+            $e = new Exception();
+            Log::warning('Avoiding segment subquery due to missing start date and/or an end date. '
+                        . 'Please ensure a start date and/or end date is set when initializing segment: '
+                        . "\n\nCreation stacktrace:\n" . $this->missingDatesException->getTraceAsString()
+                        . "\n\nUsage stacktrace:\n" . $e->getTraceAsString());
+        }
+
         $segmentExpression = $this->segmentExpression;
 
         $limitAndOffset = null;
@@ -707,7 +737,8 @@ class Segment
         }
 
         if (isset($foundStoredSegment)) {
-            return $foundStoredSegment['name'];
+            // segment name is stored sanitized
+            return Common::unsanitizeInputValues($foundStoredSegment['name']);
         }
 
         return $this->isSegmentEncoded ? urldecode($segment) : $segment;
