@@ -15,11 +15,14 @@ use Piwik\Archive;
 use Piwik\Common;
 use Piwik\Container\StaticContainer;
 use Piwik\DataTable;
+use Piwik\DataTable\DataTableInterface;
 use Piwik\DataTable\Row;
 use Piwik\Period;
 use Piwik\Period\Range;
 use Piwik\Piwik;
+use Piwik\Plugins\CoreHome\Columns\Metrics\EvolutionMetric;
 use Piwik\Plugins\Goals\Archiver;
+use Piwik\Plugins\MultiSites\Columns\Metrics\EcommerceOnlyEvolutionMetric;
 use Piwik\Plugins\SitesManager\API as APISitesManager;
 use Piwik\Scheduler\Scheduler;
 use Piwik\SettingsPiwik;
@@ -48,22 +51,25 @@ class API extends \Piwik\Plugin\API
     public const ECOMMERCE_ORDERS_METRIC = 'orders';
     public const ECOMMERCE_REVENUE_METRIC = 'ecommerce_revenue';
 
-    private static $baseMetrics = array(
-        self::NB_VISITS_METRIC   => array(
+    /** @var array<string,array<string,string>> */
+    private static $baseMetrics = [
+        self::NB_VISITS_METRIC   => [
             self::METRIC_TRANSLATION_KEY        => 'General_ColumnNbVisits',
             self::METRIC_EVOLUTION_COL_NAME_KEY => 'visits_evolution',
             self::METRIC_RECORD_NAME_KEY        => self::NB_VISITS_METRIC,
             self::METRIC_COL_NAME_KEY           => self::NB_VISITS_METRIC,
             self::METRIC_IS_ECOMMERCE_KEY       => false,
-        ),
-        self::NB_ACTIONS_METRIC  => array(
+        ],
+        self::NB_ACTIONS_METRIC  => [
             self::METRIC_TRANSLATION_KEY        => 'General_ColumnNbActions',
             self::METRIC_EVOLUTION_COL_NAME_KEY => 'actions_evolution',
             self::METRIC_RECORD_NAME_KEY        => self::NB_ACTIONS_METRIC,
             self::METRIC_COL_NAME_KEY           => self::NB_ACTIONS_METRIC,
             self::METRIC_IS_ECOMMERCE_KEY       => false,
-        )
-    );
+        ]
+    ];
+
+    protected $autoSanitizeInputParams = false;
 
     /**
      * Returns a report displaying the total visits, actions and revenue, as
@@ -81,16 +87,23 @@ class API extends \Piwik\Plugin\API
      *
      * @param string $period The period type to get data for.
      * @param string $date The date(s) to get data for.
-     * @param bool|string $segment The segments to get data for.
-     * @param bool|string $_restrictSitesToLogin Hack used to enforce we restrict the returned data to the specified username
+     * @param null|string $segment The segments to get data for.
+     * @param null|string $_restrictSitesToLogin Hack used to enforce we restrict the returned data to the specified username
      *                                        Only used when a scheduled task is running
-     * @param bool|string $enhanced When true, return additional goal & ecommerce metrics
-     * @param bool|string $pattern If specified, only the website which names (or site ID) match the pattern will be returned using SitesManager.getPatternMatchSites
-     * @param array $showColumns If specified, only the requested columns will be fetched
-     * @return DataTable
+     * @param bool $enhanced When true, return additional goal & ecommerce metrics
+     * @param null|string $pattern If specified, only the website which names (or site ID) match the pattern will be returned using SitesManager.getPatternMatchSites
+     * @param string|array<string> $showColumns If specified, only the requested columns will be fetched
+     * @return DataTableInterface
      */
-    public function getAll($period, $date, $segment = false, $_restrictSitesToLogin = false, $enhanced = false, $pattern = false, $showColumns = array())
-    {
+    public function getAll(
+        string $period,
+        string $date,
+        ?string $segment = null,
+        ?string $_restrictSitesToLogin = null,
+        bool $enhanced = false,
+        ?string $pattern = null,
+        $showColumns = []
+    ): DataTableInterface {
         Piwik::checkUserHasSomeViewAccess();
 
         $idSites = $this->getSitesIdFromPattern($pattern, $_restrictSitesToLogin);
@@ -106,14 +119,18 @@ class API extends \Piwik\Plugin\API
          *         });
          *     });
          *
-         * @param array &$idSites List of idSites that the current user would be allowed to see in all websites dashboard.
+         * @param array<int> &$idSites List of idSites that the current user would be allowed to see in all websites dashboard.
          */
         Piwik::postEvent('MultiSites.filterSites', [&$idSites]);
 
-        if (!empty($showColumns) && !is_array($showColumns)) {
+        if (!empty($showColumns) && is_string($showColumns)) {
             $showColumns = explode(',', $showColumns);
         }
 
+        if (!is_array($showColumns)) {
+            $showColumns = [];
+        }
+
         if (empty($idSites)) {
             return new DataTable();
         }
@@ -133,17 +150,13 @@ class API extends \Piwik\Plugin\API
     /**
      * Fetches the list of sites which names match the string pattern
      *
-     * @param string $pattern
-     * @param bool   $_restrictSitesToLogin
-     * @return array|string
+     * @param ?string $pattern
+     * @param ?string $_restrictSitesToLogin
+     * @return array<int>
      */
-    private function getSitesIdFromPattern($pattern, $_restrictSitesToLogin)
+    private function getSitesIdFromPattern(?string $pattern, ?string $_restrictSitesToLogin): array
     {
-        // First clear cache
-        Site::clearCache();
-
         if (empty($pattern)) {
-
             /** @var Scheduler $scheduler */
             $scheduler = StaticContainer::getContainer()->get('Piwik\Scheduler\Scheduler');
             // Then, warm the cache with only the data we should have access to
@@ -154,34 +167,29 @@ class API extends \Piwik\Plugin\API
                 // Instead, we make sure that only the right set of data is returned
                 && !$scheduler->isRunningTask()
             ) {
-                APISitesManager::getInstance()->getAllSites();
+                $sites = APISitesManager::getInstance()->getAllSites();
             } else {
-                APISitesManager::getInstance()->getSitesWithAtLeastViewAccess($limit = false, $_restrictSitesToLogin);
+                $sites = APISitesManager::getInstance()->getSitesWithAtLeastViewAccess($limit = false, $_restrictSitesToLogin);
             }
         } else {
+            /** @var array $sites */
             $sites = Request::processRequest(
                 'SitesManager.getPatternMatchSites',
-                array('pattern'   => $pattern,
-                      // added because caller could overwrite these
-                      'limit'       => SettingsPiwik::getWebsitesCountToDisplay(),
-                      'showColumns' => '',
-                      'hideColumns' => '',
-                'format'      => 'original')
+                [
+                    'pattern' => $pattern,
+                    'limit'   => SettingsPiwik::getWebsitesCountToDisplay(),
+                    'format'  => 'original'
+                ],
+                []
             );
 
             if (!empty($sites)) {
+                // cache sites for later usage
                 Site::setSitesFromArray($sites);
             }
         }
 
-        // Both calls above have called Site::setSitesFromArray. We now get these sites:
-        $sitesToProblablyAdd = Site::getSites();
-        $idSites = [];
-        foreach ($sitesToProblablyAdd as $site) {
-            $idSites[] = $site['idsite'];
-        }
-
-        return $idSites;
+        return array_column($sites, 'idsite');
     }
 
     /**
@@ -191,17 +199,23 @@ class API extends \Piwik\Plugin\API
      * @param int $idSite Id of the Matomo site
      * @param string $period The period type to get data for.
      * @param string $date The date(s) to get data for.
-     * @param bool|string $segment The segments to get data for.
-     * @param bool|string $_restrictSitesToLogin Hack used to enforce we restrict the returned data to the specified username
+     * @param null|string $segment The segments to get data for.
+     * @param null|string $_restrictSitesToLogin Hack used to enforce we restrict the returned data to the specified username
      *                                        Only used when a scheduled task is running
-     * @param bool|string $enhanced When true, return additional goal & ecommerce metrics
-     * @return DataTable
+     * @param bool $enhanced When true, return additional goal & ecommerce metrics
+     * @return DataTableInterface
      */
-    public function getOne($idSite, $period, $date, $segment = false, $_restrictSitesToLogin = false, $enhanced = false)
-    {
+    public function getOne(
+        int $idSite,
+        string $period,
+        string $date,
+        ?string $segment = null,
+        ?string $_restrictSitesToLogin = null,
+        bool $enhanced = false
+    ): DataTableInterface {
         Piwik::checkUserHasViewAccess($idSite);
 
-        $site = $this->getSiteFromId($idSite);
+        $site = APISitesManager::getInstance()->getSiteFromId($idSite);
 
         if (empty($site)) {
             return new DataTable();
@@ -215,21 +229,26 @@ class API extends \Piwik\Plugin\API
             $_restrictSitesToLogin,
             $enhanced,
             $multipleWebsitesRequested = false,
-            $showColumns = array()
+            $showColumns = []
         );
     }
 
     /**
      * @param null|string  $period
      * @param null|string  $date
-     * @param false|string $segment
+     * @param null|string $segment
      * @param string       $pattern
      * @param int          $filter_limit
-     * @return array
+     * @return array<string,mixed>
      * @throws Exception
      */
-    public function getAllWithGroups($period = null, $date = null, $segment = false, $pattern = '', $filter_limit = 0)
-    {
+    public function getAllWithGroups(
+        ?string $period = null,
+        ?string $date = null,
+        ?string $segment = null,
+        string $pattern = '',
+        int $filter_limit = 0
+    ): array {
         Piwik::checkUserHasSomeViewAccess();
 
         if (Period::isMultiplePeriod($date, $period)) {
@@ -255,15 +274,16 @@ class API extends \Piwik\Plugin\API
         return $response;
     }
 
-    private function getSiteFromId($idSite)
-    {
-        $idSite = (int) $idSite;
-        return APISitesManager::getInstance()->getSiteFromId($idSite);
-    }
-
-    private function buildDataTable($idSites, $period, $date, $segment, $_restrictSitesToLogin, $enhanced, $multipleWebsitesRequested, $showColumns)
-    {
-        // build the archive type used to query archive data
+    private function buildDataTable(
+        array $idSites,
+        string $period,
+        string $date,
+        ?string $segment,
+        ?string $_restrictSitesToLogin,
+        bool $enhanced,
+        bool $multipleWebsitesRequested,
+        ?array $showColumns
+    ): DataTableInterface {
         $archive = Archive::build(
             $idSites,
             $period,
@@ -273,9 +293,9 @@ class API extends \Piwik\Plugin\API
         );
 
         // determine what data will be displayed
-        $fieldsToGet = array();
-        $columnNameRewrites = array();
-        $apiECommerceMetrics = array();
+        $fieldsToGet = [];
+        $columnNameRewrites = [];
+        $apiECommerceMetrics = [];
         $apiMetrics = API::getApiMetrics($enhanced);
         foreach ($apiMetrics as $metricName => $metricSettings) {
             if (!empty($showColumns) && !in_array($metricName, $showColumns)) {
@@ -290,12 +310,13 @@ class API extends \Piwik\Plugin\API
             }
         }
 
+        /** @var DataTable|DataTable\Map $dataTable */
         $dataTable = $archive->getDataTableFromNumericAndMergeChildren($fieldsToGet);
 
         $this->populateLabel($dataTable);
 
         // replace record names with user friendly metric names
-        $dataTable->filter('ReplaceColumnNames', array($columnNameRewrites));
+        $dataTable->filter('ReplaceColumnNames', [$columnNameRewrites]);
 
         $totalMetrics = $this->preformatApiMetricsForTotalsCalculation($apiMetrics);
         $this->setMetricsTotalsMetadata($dataTable, $totalMetrics);
@@ -314,7 +335,7 @@ class API extends \Piwik\Plugin\API
 
             $pastArchive = Archive::build($idSites, $period, $strLastDate, $segment, $_restrictSitesToLogin);
             $pastData = $pastArchive->getDataTableFromNumericAndMergeChildren($fieldsToGet);
-            $pastData->filter('ReplaceColumnNames', array($columnNameRewrites));
+            $pastData->filter('ReplaceColumnNames', [$columnNameRewrites]);
             $this->populateLabel($pastData); // labels are needed to calculate evolution
             $this->calculateEvolutionPercentages($dataTable, $pastData, $apiMetrics);
             $this->setPreviousMetricsTotalsMetadata($dataTable, $pastData, $totalMetrics);
@@ -326,29 +347,35 @@ class API extends \Piwik\Plugin\API
         }
 
         // move the site id to a metadata column
-        $dataTable->queueFilter('MetadataCallbackAddMetadata', array('idsite', 'group', function ($idSite) {
-            if ($idSite == '-1') { // Others row might occur when `filter_truncate` API parameter is used
-                return '';
-            }
-            return Site::getGroupFor($idSite);
-        }, array()));
-        $dataTable->queueFilter('MetadataCallbackAddMetadata', array('idsite', 'main_url', function ($idSite) {
-            if ($idSite == '-1') { // Others row might occur when `filter_truncate` API parameter is used
-                return '';
-            }
-            return Site::getMainUrlFor($idSite);
-        }, array()));
+        $dataTable->queueFilter('MetadataCallbackAddMetadata', [
+            'idsite', 'group', function ($idSite) {
+                if ($idSite == '-1') { // Others row might occur when `filter_truncate` API parameter is used
+                    return '';
+                }
+                return Site::getGroupFor($idSite);
+            }, []
+        ]);
+        $dataTable->queueFilter('MetadataCallbackAddMetadata', [
+            'idsite', 'main_url', function ($idSite) {
+                if ($idSite == '-1') { // Others row might occur when `filter_truncate` API parameter is used
+                    return '';
+                }
+                return Site::getMainUrlFor($idSite);
+            }, []
+        ]);
 
         // set the label of each row to the site name
         if ($multipleWebsitesRequested) {
-            $dataTable->queueFilter('ColumnCallbackReplace', array('label', function ($idSite) {
-                if ($idSite == '-1') { // Others row might occur when `filter_truncate` API parameter is used
-                    return Piwik::translate('General_Others');
+            $dataTable->queueFilter('ColumnCallbackReplace', [
+                'label', function ($idSite) {
+                    if ($idSite == '-1') { // Others row might occur when `filter_truncate` API parameter is used
+                        return Piwik::translate('General_Others');
+                    }
+                    return Site::getNameFor($idSite);
                 }
-                return Site::getNameFor($idSite);
-            }));
+            ]);
         } else {
-            $dataTable->queueFilter('ColumnDelete', array('label'));
+            $dataTable->queueFilter('ColumnDelete', ['label']);
         }
 
         // filter rows without visits
@@ -393,15 +420,20 @@ class API extends \Piwik\Plugin\API
      *
      * @param DataTable|DataTable\Map $currentData
      * @param DataTable|DataTable\Map $pastData
-     * @param array $apiMetrics The array of string fields to calculate evolution
-     *                          metrics for.
+     * @param array<string,array<string,string>> $apiMetrics The array of string fields to calculate evolution metrics for.
      * @throws Exception
      */
-    private function calculateEvolutionPercentages($currentData, $pastData, $apiMetrics)
-    {
+    private function calculateEvolutionPercentages(
+        DataTableInterface $currentData,
+        DataTableInterface $pastData,
+        array $apiMetrics
+    ): void {
         if (get_class($currentData) != get_class($pastData)) { // sanity check for regressions
-            throw new Exception("Expected \$pastData to be of type " . get_class($currentData) . " - got "
-                . get_class($pastData) . ".");
+            throw new Exception(sprintf(
+                'Expected $pastData to be of type %1$s - got %2$s.',
+                get_class($currentData),
+                get_class($pastData)
+            ));
         }
 
         if ($currentData instanceof DataTable\Map) {
@@ -414,8 +446,8 @@ class API extends \Piwik\Plugin\API
             $extraProcessedMetrics = $currentData->getMetadata(DataTable::EXTRA_PROCESSED_METRICS_METADATA_NAME);
             foreach ($apiMetrics as $metricSettings) {
                 $evolutionMetricClass = $this->isEcommerceEvolutionMetric($metricSettings)
-                                      ? "Piwik\\Plugins\\MultiSites\\Columns\\Metrics\\EcommerceOnlyEvolutionMetric"
-                                      : "Piwik\\Plugins\\CoreHome\\Columns\\Metrics\\EvolutionMetric";
+                    ? EcommerceOnlyEvolutionMetric::class
+                    : EvolutionMetric::class;
 
                 $extraProcessedMetrics = is_array($extraProcessedMetrics) ? $extraProcessedMetrics : [];
                 $extraProcessedMetrics[] = new $evolutionMetricClass(
@@ -433,73 +465,73 @@ class API extends \Piwik\Plugin\API
     /**
      * @ignore
      */
-    public static function getApiMetrics($enhanced)
+    public static function getApiMetrics(bool $enhanced): array
     {
         $metrics = self::$baseMetrics;
 
         if (Common::isActionsPluginEnabled()) {
-            $metrics[self::NB_PAGEVIEWS_LABEL] = array(
+            $metrics[self::NB_PAGEVIEWS_LABEL] = [
                 self::METRIC_TRANSLATION_KEY        => 'General_ColumnPageviews',
                 self::METRIC_EVOLUTION_COL_NAME_KEY => 'pageviews_evolution',
                 self::METRIC_RECORD_NAME_KEY        => self::NB_PAGEVIEWS_METRIC,
                 self::METRIC_COL_NAME_KEY           => self::NB_PAGEVIEWS_LABEL,
                 self::METRIC_IS_ECOMMERCE_KEY       => false,
-            );
-            $metrics[self::NB_HITS_LABEL] = array(
+            ];
+            $metrics[self::NB_HITS_LABEL] = [
                 self::METRIC_TRANSLATION_KEY        => 'General_ColumnHits',
                 self::METRIC_EVOLUTION_COL_NAME_KEY => 'hits_evolution',
                 self::METRIC_RECORD_NAME_KEY        => self::NB_HITS_METRIC,
                 self::METRIC_COL_NAME_KEY           => self::NB_HITS_LABEL,
                 self::METRIC_IS_ECOMMERCE_KEY       => false,
-            );
+            ];
         }
 
         if (Common::isGoalPluginEnabled()) {
             // goal revenue metric
-            $metrics[self::GOAL_REVENUE_METRIC] = array(
+            $metrics[self::GOAL_REVENUE_METRIC] = [
                 self::METRIC_TRANSLATION_KEY        => 'General_ColumnRevenue',
                 self::METRIC_EVOLUTION_COL_NAME_KEY => self::GOAL_REVENUE_METRIC . '_evolution',
                 self::METRIC_RECORD_NAME_KEY        => Archiver::getRecordName(self::GOAL_REVENUE_METRIC),
                 self::METRIC_COL_NAME_KEY           => self::GOAL_REVENUE_METRIC,
                 self::METRIC_IS_ECOMMERCE_KEY       => false,
-            );
+            ];
 
             if ($enhanced) {
                 // number of goal conversions metric
-                $metrics[self::GOAL_CONVERSION_METRIC] = array(
+                $metrics[self::GOAL_CONVERSION_METRIC] = [
                     self::METRIC_TRANSLATION_KEY        => 'Goals_ColumnConversions',
                     self::METRIC_EVOLUTION_COL_NAME_KEY => self::GOAL_CONVERSION_METRIC . '_evolution',
                     self::METRIC_RECORD_NAME_KEY        => Archiver::getRecordName(self::GOAL_CONVERSION_METRIC),
                     self::METRIC_COL_NAME_KEY           => self::GOAL_CONVERSION_METRIC,
                     self::METRIC_IS_ECOMMERCE_KEY       => false,
-                );
+                ];
 
                 // number of orders
-                $metrics[self::ECOMMERCE_ORDERS_METRIC] = array(
+                $metrics[self::ECOMMERCE_ORDERS_METRIC] = [
                     self::METRIC_TRANSLATION_KEY        => 'General_EcommerceOrders',
                     self::METRIC_EVOLUTION_COL_NAME_KEY => self::ECOMMERCE_ORDERS_METRIC . '_evolution',
                     self::METRIC_RECORD_NAME_KEY        => Archiver::getRecordName(self::GOAL_CONVERSION_METRIC, 0),
                     self::METRIC_COL_NAME_KEY           => self::ECOMMERCE_ORDERS_METRIC,
                     self::METRIC_IS_ECOMMERCE_KEY       => true,
-                );
+                ];
 
                 // eCommerce revenue
-                $metrics[self::ECOMMERCE_REVENUE_METRIC] = array(
+                $metrics[self::ECOMMERCE_REVENUE_METRIC] = [
                     self::METRIC_TRANSLATION_KEY        => 'General_ProductRevenue',
                     self::METRIC_EVOLUTION_COL_NAME_KEY => self::ECOMMERCE_REVENUE_METRIC . '_evolution',
                     self::METRIC_RECORD_NAME_KEY        => Archiver::getRecordName(self::GOAL_REVENUE_METRIC, 0),
                     self::METRIC_COL_NAME_KEY           => self::ECOMMERCE_REVENUE_METRIC,
                     self::METRIC_IS_ECOMMERCE_KEY       => true,
-                );
+                ];
             }
         }
 
         return $metrics;
     }
 
-    private function preformatApiMetricsForTotalsCalculation($apiMetrics)
+    private function preformatApiMetricsForTotalsCalculation(array $apiMetrics): array
     {
-        $metrics = array();
+        $metrics = [];
         foreach ($apiMetrics as $label => $metricsInfo) {
             $totalMetadataName = self::getTotalMetadataName($label);
             $metrics[$totalMetadataName] = $metricsInfo[self::METRIC_COL_NAME_KEY];
@@ -512,18 +544,13 @@ class API extends \Piwik\Plugin\API
      * Sets the total visits, actions & revenue for a DataTable returned by
      * $this->buildDataTable.
      *
-     * @param DataTable $dataTable
-     * @param array $apiMetrics Metrics info.
-     * @return array Array of three values: total visits, total actions, total revenue
+     * @param DataTableInterface $dataTable
+     * @param array<string,string> $apiMetrics Metrics info.
      */
-    private function setMetricsTotalsMetadata($dataTable, $apiMetrics)
+    private function setMetricsTotalsMetadata(DataTableInterface $dataTable, array $apiMetrics): void
     {
-        if ($dataTable instanceof DataTable\Map) {
-            foreach ($dataTable->getDataTables() as $table) {
-                $this->setMetricsTotalsMetadata($table, $apiMetrics);
-            }
-        } else {
-            $totals = array();
+        $dataTable->filter(function (DataTable $dataTable) use ($apiMetrics) {
+            $totals = [];
             foreach ($apiMetrics as $label => $recordName) {
                 $totals[$label] = 0;
             }
@@ -539,19 +566,22 @@ class API extends \Piwik\Plugin\API
             }
 
             $dataTable->setMetadataValues($totals);
-        }
+        });
     }
 
     /**
      * Sets the previous total visits, actions & revenue for a DataTable returned by
      * $this->buildDataTable.
      *
-     * @param DataTable $dataTable
-     * @param array $apiMetrics Metrics info.
-     * @return array Array of three values: total visits, total actions, total revenue
+     * @param DataTable|DataTable\Map $dataTable
+     * @param DataTable|DataTable\Map $pastData
+     * @param array<string,string> $apiMetrics Metrics info.
      */
-    private function setPreviousMetricsTotalsMetadata($dataTable, $pastData, $apiMetrics)
-    {
+    private function setPreviousMetricsTotalsMetadata(
+        DataTableInterface $dataTable,
+        DataTableInterface $pastData,
+        array $apiMetrics
+    ): void {
         if ($dataTable instanceof DataTable\Map) {
             $currentDataTables = $dataTable->getDataTables();
             $pastDataTables = $pastData->getDataTables();
@@ -566,7 +596,7 @@ class API extends \Piwik\Plugin\API
                 );
             }
         } else {
-            $totals = array();
+            $totals = [];
             foreach ($apiMetrics as $label => $recordName) {
                 $label = 'previous_' . $label;
 
@@ -590,9 +620,9 @@ class API extends \Piwik\Plugin\API
 
     /**
      * @param Row[] $rows
-     * @return mixed
+     * @return Row[]
      */
-    private function filterRowsForTotalsCalculation($rows)
+    private function filterRowsForTotalsCalculation(array $rows): array
     {
         /**
          * Triggered to filter / restrict which rows should be included in the MultiSites (All Websites Dashboard)
@@ -611,42 +641,38 @@ class API extends \Piwik\Plugin\API
          *
          * @param Row[] &$rows An array containing rows, one row for each site. The label columns equals the idSite.
          */
-        Piwik::postEvent('MultiSites.filterRowsForTotalsCalculation', array(&$rows));
+        Piwik::postEvent('MultiSites.filterRowsForTotalsCalculation', [&$rows]);
 
         return $rows;
     }
 
-    private static function getTotalMetadataName($name)
+    private static function getTotalMetadataName(string $name): string
     {
         return 'total_' . $name;
     }
 
-    private static function getLastPeriodMetadataName($name)
+    private static function getLastPeriodMetadataName(string $name): string
     {
         return 'last_period_' . $name;
     }
 
-    private function populateLabel($dataTable)
+    private function populateLabel(DataTableInterface $dataTable): void
     {
+        // ensure label column is set and always the first column
         $dataTable->filter(function (DataTable $table) {
             foreach ($table->getRowsWithoutSummaryRow() as $row) {
                 $row->setColumn('label', $row->getMetadata('idsite'));
-            }
-        });
-        // make sure label column is always first column
-        $dataTable->queueFilter(function (DataTable $table) {
-            foreach ($table->getRowsWithoutSummaryRow() as $row) {
-                $row->setColumns(array_merge(array('label' => $row->getColumn('label')), $row->getColumns()));
+                $row->setColumns(array_merge(['label' => $row->getColumn('label')], $row->getColumns()));
             }
         });
     }
 
-    private function isEcommerceEvolutionMetric($metricSettings)
+    private function isEcommerceEvolutionMetric(array $metricSettings): bool
     {
-        return in_array($metricSettings[self::METRIC_EVOLUTION_COL_NAME_KEY], array(
+        return in_array($metricSettings[self::METRIC_EVOLUTION_COL_NAME_KEY], [
             self::GOAL_REVENUE_METRIC . '_evolution',
             self::ECOMMERCE_ORDERS_METRIC . '_evolution',
             self::ECOMMERCE_REVENUE_METRIC . '_evolution'
-        ));
+        ]);
     }
 }
